-- ##############################################################
-- # (C) 2016 BAE SYSTEMS
-- #
-- # ECOA Trad_v_ECOA Sample - "Traditional" Ada Implementation - Interface Package Body
-- #
-- # $Id: TradIF.adb,v 2.1 2017/11/06 15:33:36 ksappleb Exp $
-- #
-- # Part of the BAE Systems, Rochester, ECOA Samples collection
-- #
-- # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
-- #
-- # This software is developed for and on behalf of BAE Systems (Operations)
-- # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
-- GE Aviation Systems Limited,
-- # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
-- # and the copyright is owned by BAE Systems (Operations) Limited,
-- # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
-- # GE Aviation Systems Limited, General Dynamics United Kingdom
-- # Limited and Leonardo MW Ltd
-- #
-- # This software is developed by BAE Systems (Operations)
-- # Limited, Electronic Systems, and is the Intellectual
-- # Property of BAE Systems (Operations) Limited, Electronic Systems.
-- #
-- # The information set out in this document is provided solely
-- # on an 'as is' basis and the co-developers of this software make
-- # no warranties expressed or implied, including no warranties
-- # as to completeness, accuracy or fitness for purpose, with respect
-- # to any of the information.
-- ##############################################################

with Text_IO, System, System.Storage_Elements, Location;
 use System.Storage_Elements;
package body TradIF is

	-- ADC channel numbers
	ALTIM_CHAN : constant := 1;
	SPEED_CHAN : constant := 2;
	
	-- ADC hardware addresses
	ADCBase    : constant := 16#F000#;
	ADCAddr    : array( 1..4 ) of System.Address := ( System.Storage_Elements.To_Address(ADCBase + 16#10#),
													  System.Storage_Elements.To_Address(ADCBase + 16#20#),
													  System.Storage_Elements.To_Address(ADCBase + 16#30#),
													  System.Storage_Elements.To_Address(ADCBase + 16#40#) );
	-- Data bus port numbers
	POSN_PORT  : constant := 21;
	WP_PORT    : constant := 58;
	
	-- DAC hardware address
	DACAddr    : constant := 16#ABCD#;
	
	------------------------------------------------------------------------------------------------

	package lastGood is
		acAlt, acSpeed : integer;
		acPosn, wp     : Location.Location;
	end lastGood;
	
	------------------------------------------------------------------------------------------------
	
-- ****************************************************************
-- Comments marked "--@" are code that has specific dependencies
-- not relevant, and therefore masked, for this demonstration...
-- ****************************************************************

	-- Mimics reading a block of stuff from a serial / network channel... 
	procedure readBlocking( port : in integer; msgBufferRef : in System.Address; bufLength : in integer; timeOut : in duration; msgLength : out integer; isOK : out boolean ) is
	begin
		if timeout > 0.0 then
			-- Read data bus port or time out...
--@			DataBus.setTimeOut( port, timeout );
--@			msgLength := DataBus.Receive( busAddress, port, msgBufferRef, bufLength );
			null;
		else
			-- Read bus port if data available...
--@			if DataBus.hasData( port ) then
--@				msgLength := DataBus.Receive( busAddress, port, msgBufferRef, bufLength );
--@			else
				msgLength := 0;
--@			end if;
		end if;
		--
		if msgLength > 0 then
			isOK := true;
		else
			isOK := false;
		end if;
	end readBlocking;
	
	------------------------------------------------------------------------------------------------
	
	procedure readADC( chan : in integer; result : out integer; isOK : out boolean ) is
		ADCInput : integer := 0;
--@			for ADCInput'Address use ADCAddr( chan );  -- don't try this at home children...!
		ADCIsReady : boolean := true;
--@			for ADCIsReady'Address use System.Storage_Elements.To_Address(ADCBase);
	begin
		if ADCIsReady then
			-- read the ADC channel chan
			result := ADCInput;
		else
			-- return a dummy result
			result := 0;
		end if;
		isOK := ADCIsReady;
	end readADC;
	
	------------------------------------------------------------------------------------------------
	
	procedure getAlt( acAlt : out integer ) is
		value : integer;
		readOK : boolean;
	begin
		readADC( ALTIM_CHAN, value, readOK );
		if readOK then
			acAlt := value;
			lastGood.acAlt := value;
		else
--@			Text_IO.put_line( "error: ADC not ready" );
--@			Text_IO.put_line( "using last good value for acAlt" );
			acAlt := lastGood.acAlt;
		end if;
	end getAlt;
		
	------------------------------------------------------------------------------------------------
	
	procedure getSpeed( acSpeed : out integer ) is
		value : integer;
		readOK : boolean;
	begin
		readADC( SPEED_CHAN, value, readOK );
		if readOK then
			acSpeed := value;
			lastGood.acSpeed := value;
		else
--@			Text_IO.put_line( "error: ADC not ready" );
--@			Text_IO.put_line( "using last good value for acSpeed" );
			acSpeed := lastGood.acSpeed;
		end if;
	end getSpeed;
		
	------------------------------------------------------------------------------------------------
	
	procedure getPosn( acPosn : out Location.Location ) is
		value  : Location.Location;
		msgLen : integer;
		readOK : boolean;
	begin
		Location.set( acPosn, 0, 0 );
		--
		readBlocking( POSN_PORT, value'address, value'size, 0.0, msgLen, readOK ); -- Polled operation. timeout = 0.0
		if readOK then
			acPosn := value;
			lastGood.acPosn := value;
		else
--@			Text_IO.put_line( "error: read of bus port " & integer'image( POSN_PORT ) & " timedout" );
--@			Text_IO.put_line( "using last good value for acPosn" );
			acPosn := lastGood.acPosn;
		end if;
	end getPosn;
		
	------------------------------------------------------------------------------------------------
	
	procedure getWayPoint( wp : out Location.Location ) is
		value  : Location.Location;
		msgLen : integer;
		readOK : boolean;
	begin
		Location.set( wp, 0, 0 );
		--
		readBlocking( WP_PORT, value'address, value'size, 0.0, msgLen, readOK ); -- Polled operation. timeout = 0.0
		if readOK then
			wp := value;
			lastGood.wp := value;
		else
			-- We don't care if this one misses ...
			wp := lastGood.wp;
		end if;
	end getWayPoint;
	
	------------------------------------------------------------------------------------------------
	
	procedure putSteer( steer : in integer ) is
		DACOutput : integer;
--@			for DACOutput'Address use System.Storage_Elements.To_Address(DACAddr);  -- don't try this at home children...!
	begin
		-- Output the steer setting to the control surface hardware
		DACOutput := steer;
	end putSteer;
	
end TradIF;
